module resummation
  use types; use consts_dp
  use rad_tools
  use pdfs_tools
  use warnings_and_errors
  use emsn_tools
  implicit none

  private
  public :: resummed_sigma

contains
  !======================================================================
  function resummed_sigma(pt, cs, order,dlumi_lumi) result(sigma)
    real(dp),                  intent(in) :: pt(:)
    type(process_and_parameters), intent(in) :: cs
    integer,                   intent(in) :: order
    real(dp), intent(out), optional       :: dlumi_lumi(size(pt))
    real(dp)  :: sigma(size(pt))
    !------------------------------
    real(dp) :: L_tilde(size(pt)), lambda(size(pt)), lNLL(size(pt)), lNNLL(size(pt))
    real(dp) :: normalisation

    L_tilde = Ltilde(pt/cs%Q, cs%p)
    lambda  = get_lambda(L_tilde, cs)

    sigma = zero
    where (lambda < half)  sigma = exp(Rad(L_tilde, cs, order))

    if (present(dlumi_lumi))  dlumi_lumi = zero    

    if (order == order_LL) then
       sigma = sigma * lumi_LL(cs)
    else if (order == order_NLL) then
       sigma = sigma * lumi_NLL(exp(-L_tilde)*cs%muF, cs)
    else
       if (order /= order_NNLL) call wae_error("expected order_NNLL, found", intval=order)

       ! include the non-inclusive correction
       sigma = sigma * (1 + &
            &          non_incl(cs%jet_radius,'all') * Rad_p(lambda)*two*cs%as2pi/(1-two*lambda))

       ! include the hard correction and the NNLL luminosity
       lNLL  = lumi_NLL(exp(-L_tilde)*cs%muF, cs)
       lNNLL = lumi_NNLL(exp(-L_tilde)*cs%muF, lambda, cs)
       if (cs%include_c1_squared) then
          ! Option 1 for luminosity: everything factorised
          sigma = sigma * (1+cs%as2pi*H(1))* lNNLL

          if (present(dlumi_lumi)) then
             dlumi_lumi = (1+cs%as2pi*H(1))* lNNLL/lNLL-one
          end if
       else
          ! Option 2 for luminosity: no spurious O(as^2) terms.
          ! This is the option that we take as our default.
          sigma = sigma * ( lNNLL+ lNLL*cs%as2pi*H(1) )
          if (present(dlumi_lumi)) then
             dlumi_lumi = lNNLL/lNLL+ cs%as2pi*H(1)-one
          end if
       end if
    end if
  end function resummed_sigma

end module resummation
